@@ -142,7 +142,7 @@ class KeyWhichResponse:
         return r
 
     def unmarshal_json(self, j: dict):
-        if "Key" in j:
+        if "Key" in j and j["Key"] is not None:
             self.key = Key.from_json(j["Key"])
         if "Error" in j:
             self.error = j["Error"]
@@ -203,7 +203,7 @@ class KeyTempResponse:
         return r
 
     def unmarshal_json(self, j: dict):
-        if "Key" in j:
+        if "Key" in j and j["Key"] is not None:
             self.key = Key.from_json(j["Key"])
         if "Private" in j:
             self.private = j["Private"]
@@ -625,7 +625,7 @@ class DirLookupResponse:
         return r
 
     def unmarshal_json(self, j: dict):
-        if "Entry" in j:
+        if "Entry" in j and j["Entry"] is not None:
             self.entry = DirEntry.from_json(j["Entry"])
         if "Error" in j:
             self.error = j["Error"]
