@@ -5,6 +5,7 @@ from fsspec import spec  # type: ignore
 from spinsrv import spin
 from spinsrv import spinpy
 
+
 @dataclasses.dataclass
 class Config:
     public: str = ""
@@ -12,6 +13,8 @@ class Config:
     citizen: spin.CitizenName = ""
 
 
+# I copied in a sample from the Databricks filesystem implementation
+# https://github.com/fsspec/filesystem_spec/blob/master/fsspec/implementations/dbfs.py
 class SpinFileSystem(spec.AbstractFileSystem):
     protocol = "spin"
     root_marker = "/"
@@ -53,7 +56,8 @@ class SpinFileSystem(spec.AbstractFileSystem):
                     level=1,
                 )
             )
-            # TODO check error
+            if resp.error != "":
+                raise OSError(resp.error)  # TODO: better error handling?
             entries = resp.entries
             out = [
                 {
@@ -69,6 +73,7 @@ class SpinFileSystem(spec.AbstractFileSystem):
             return out
         return [o["name"] for o in out]
 
+    # helper to get parents of a path, used by makedirs below
     def _parents(self, path):
         x = path.split("/")
         ps = []
@@ -89,7 +94,6 @@ class SpinFileSystem(spec.AbstractFileSystem):
             exists before creating it (and raises an
             Exception if this is the case)
         """
-
         ops = []
         ps = self._parents(path)
         for p in ps:
@@ -116,9 +120,11 @@ class SpinFileSystem(spec.AbstractFileSystem):
                 ops=ops,
             )
         )
+        if resp.error != "":
+            raise OSError(resp.error)  # TODO: better error handling?
 
-        # TODO what to do about this?
-        self.invalidate_cache(self._parent(path))
+        for p in ps:
+            self.invalidate_cache(p)
 
     def mkdir(self, path, create_parents=True, **kwargs):
         """
@@ -176,12 +182,14 @@ class SpinFileSystem(spec.AbstractFileSystem):
                 ],
             )
         )
+        if resp.error != "":
+            raise OSError(resp.error)  # TODO: better error handling?
         # TODO check for error on resp
         self.invalidate_cache(self._parent(path))
 
     def _open(self, path, mode="rb", block_size="default", **kwargs):
         """
-        Overwrite the base class method to make sure to create a DBFile.
+        Overwrite the base class method to make sure to create a SpinFile.
         All arguments are copied from the base method.
         """
         return SpinFile(self, path, mode=mode, block_size=block_size, **kwargs)
@@ -197,7 +205,7 @@ class SpinFileSystem(spec.AbstractFileSystem):
             )
         )
         if resp.error != "":
-            raise OSError(resp.error)
+            raise OSError(resp.error)  # TODO: better handling?
         return resp.entry
 
     # get the data from the store
@@ -216,7 +224,8 @@ class SpinFileSystem(spec.AbstractFileSystem):
             )
         )
         if resp.error != "":
-            raise OSError(resp.error)
+            print(resp)
+            raise OSError(resp.error)  # TODO: better handling
         if len(resp.outcomes) != 1:
             print(resp)
             raise OSError(f"expected 1 outcome, got {len(resp.outcomes)}")
@@ -231,6 +240,8 @@ class SpinFileSystem(spec.AbstractFileSystem):
         super().invalidate_cache(path)
 
 
+# I copied in a sample from the Databricks filesystem implementation
+# https://github.com/fsspec/filesystem_spec/blob/master/fsspec/implementations/dbfs.py
 class SpinFile(spec.AbstractBufferedFile):
     """
     Helper class for files referenced in the SpinFileSystem.
@@ -287,7 +298,9 @@ class SpinFile(spec.AbstractBufferedFile):
             data[start:end] for start, end in self._to_sized_blocks(len(data))
         ]
 
-        for (i, data_chunk) in enumerate(data_chunks):
+        import tqdm
+
+        for (i, data_chunk) in tqdm.tqdm(enumerate(data_chunks), total=len(data_chunks)):
             resp = self.fs.bc.apply(
                 spin.BitApplyRequest(
                     public=self.fs.config.public,
@@ -297,7 +310,8 @@ class SpinFile(spec.AbstractBufferedFile):
             )
 
             if resp.error != "":
-                raise OSError(resp.error)
+                print(resp)
+                raise OSError(resp.error)  # TODO: better handling?
 
             if len(resp.outcomes) != 1:
                 print(resp)
@@ -312,7 +326,6 @@ class SpinFile(spec.AbstractBufferedFile):
             )
 
         if final:
-            # TODO check for error
             resp = self.fs.dc.apply(
                 spin.DirApplyRequest(
                     public=self.fs.config.public,
@@ -326,23 +339,26 @@ class SpinFile(spec.AbstractBufferedFile):
             )
             if resp.error != "":
                 print(resp)
-                raise OSError(resp.error)
+                raise OSError(resp.error)  # TODO: better handling
             self.dir_entry = resp.entries[0]
             self.fs.invalidate_cache(self.fs._parent(self.dir_entry.path))
             return True
 
+    # warn: this code is not seriously tested
     def _intersection(self, a, b, c, d):
-        # compute [a, b] \cap [c,d]
-        if b < c:
+        # compute [a, b] ∩ [c,d]
+        if b < c or a > d:
             return (0, 0)
         return (max(a, c), min(b, d))
 
+    # warn: this code is not seriously tested
     def _blocks_in_range(self, start, end):
+        # find the blocks for the file for the bit range start to end
         out = []
         for b in self.dir_entry.blocks:
             bstart, bend = b.offset, b.offset + b.size
             istart, iend = self._intersection(start, end, bstart, bend)
-            if iend - istart == 0:
+            if iend - istart == 0:  # skip if empty intersection
                 continue
             out.append((b, (istart, iend)))
         return out
